	// Including libnds’ set of defines
#include <nds.h>
	// Include basic memory functions
#include <string.h>

	// Including our converted graphics
#include "todHotelTiles_bin.h"
#include "todHotelMap_bin.h"
#include "todHotelPal_bin.h"
		
#include "todLogoRaw_bin.h"
#include "todLogoPal_bin.h"
		
int main()
{
		// Powering up the DS 2D hardware
	REG_POWERCNT = POWER_ALL_2D;
		
		// Enable Vertical blank interrupt for waitForVBlank BIOS call
	irqInit();
	irqEnable(IRQ_VBLANK);
		
		// Setting up video mode to mode 5:
		// Background layer 0 will be used as tiled background
		// Background layer 3 will be used as 8bit bitmap layer
		// Note the EXT_PALETTE setup for backgrounds
	DISPLAY_CR = MODE_5_2D | DISPLAY_BG0_ACTIVE | DISPLAY_BG3_ACTIVE | DISPLAY_BG_EXT_PALETTE;
		
		// Setup VRAM bank A to use for background graphics (128KB)
	VRAM_A_CR = VRAM_ENABLE | VRAM_A_MAIN_BG;
		
		// Now, setup the text background using extended palette
	BG0_CR = BG_32x32 | BG_COLOR_256 | BG_MAP_BASE(0) | BG_TILE_BASE(1) | BG_PRIORITY(1);
		
		// Setup the bitmap layer as well, on top of the text BG
	BG3_CR = BG_BMP8_256x256 | BG_BMP_BASE(3) | BG_PRIORITY(0);
		
		// Copy the map to the background's map base
	memcpy((void*)BG_MAP_RAM(0), todHotelMap_bin, todHotelMap_bin_size);
		// Copy the tiles to background's tilebase
	memcpy((void*)BG_TILE_RAM(1), todHotelTiles_bin, todHotelTiles_bin_size);
		// Copy the bitmap data to background's BMP base
	memcpy((void*)BG_BMP_RAM(3), todLogoRaw_bin, todLogoRaw_bin_size);
		
		// Map VRAM F to CPU space
	VRAM_F_CR = VRAM_ENABLE | VRAM_F_LCD;
		
		// Copy our palettes to each BG's bank
		// Copy text BG layer (BG0) to bank 0
	memcpy(VRAM_F_EXT_PALETTE[0], todHotelPal_bin, todHotelPal_bin_size);
		// Copy the bitmap palette to it's normal location
	memcpy(BG_PALETTE, todLogoPal_bin, todLogoPal_bin_size);
		
		// Map VRAM F to where it belongs
	VRAM_F_CR = VRAM_ENABLE | VRAM_F_BG_EXT_PALETTE;
		
		// Setup first time affine matrix params
		// Default matrix given at boot is illegal, resulting a hidden background
		// This is a _must_ if you just want to display a bitmap background which does not rotate
	BG3_XDX = 1 << 8;
	BG3_YDY = 1 << 8;
		
		// Used to have a nice smooth sine movement
	u32 sineCounter = 0;
		
		// Infinate loop the keep the demo running
	while(1) {
			
			// Use an angle from sine look-up table to get a nice rotation value
		const s8 tempAngle = SIN[sineCounter & 0x1FF] >> 8;
			// Get sinus and cosinus values from a pre-calculated look-up table
		const s16 sinVal = SIN[ tempAngle & 0x1FF ] >> 4;
		const s16 cosVal = COS[ tempAngle & 0x1FF ] >> 4;
			// Write background's affine matrix registers
		BG3_XDX =  cosVal;
		BG3_XDY = -sinVal;
		BG3_YDX =  sinVal;
		BG3_YDY =  cosVal;
			// Setting rotation center to 128,128 - middle of background
		BG3_CX = (128 << 8) - 128 * (cosVal - sinVal);
		BG3_CY = (128 << 8) - 128 * (sinVal + cosVal);
			
			// Increase the sine counter for next frame
		sineCounter += 4;
			
			// Rest untill next frame
		swiWaitForVBlank();
	}
}
